<?php

namespace OrganizationsManagement\Service;

use Exception;
use OrganizationsManagement\Util\Messages;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\InstanceProvider;
use Doctrine\ORM\EntityManager;

class Slacliente
{
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var \Teknisa\Model\Teknisa\Repositories\Slacliente
     */
    private $slaclienteRepository;
    /**
     * @var \OrganizationsManagement\Factory\Slacliente
     */
    private $slaclienteFactory;

    /**
     * SlaclienteParam constructor.
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
        $this->slaclienteRepository = InstanceProvider::getRepository()->getSlacliente();
        $this->slaclienteFactory = InstanceProvider::getFactory()->getSlacliente();
    }

    /**
     * @param $cdcliente
     * @return array
     */
    public function findSlacliente($cdcliente): array
    {
        return InstanceProvider::getRepository()->getSlacliente()->findSlacliente($cdcliente);
    }

    /**
     * @param $rows
     * @throws Exception
     */
    public function updateSlacliente($rows)
    {
        $slaclienteRepository = InstanceProvider::getRepository()->getSlacliente();
        $slaclienteFactory = InstanceProvider::getFactory()->getSlacliente();
        $environment = InstanceProvider::getLibEnvironment();
        $nrorg = $environment->getCurrentOrganizationId();
        $currentOperator = $environment->getCurrentUserId();
        $dtAtual = new \DateTime();
        try {
            foreach ($rows as $row) {
                $filter = array(
                    'cdcliente' => $row['CDCLIENTE'],
                    'cdgrupfase' => $row['CDGRUPFASE'],
                    "cdassunto" => $row['CDASSUNTO'],
                    "cdcomplexid" => $row['CDCOMPLEXID']
                );
                /** @var \Teknisa\Model\Teknisa\Entities\Slacliente $slacliente */
                $slacliente = $slaclienteRepository->findOneBy($filter);

                $dtiniciosla = empty($row['DTINICIOSLA']) ? null : $dtAtual::createFromFormat('Y-m-d', $row['DTINICIOSLA']);
                $qtdiassla = empty($row['QTDIASSLA']) ? 0 : $row['QTDIASSLA'] / 8;

                $slacliente = $slaclienteFactory->update($slacliente, $row['IDPRAZOSLA'], $qtdiassla, $nrorg,
                    $currentOperator, $dtAtual, 'S', $dtiniciosla);
                $this->entityManager->persist($slacliente);
            }
            $this->entityManager->flush();
        } catch(Exception $e) {
            throw new Exception($e->getMessage());
        }
    }

    /**
     * @param $cdcliente
     * @param $cdassunto
     * @param $cdgrupfase
     * @param $idprazosla
     * @param $qtdiassla
     * @param $dtiniciosla
     * @throws Exception
     */
    public function saveSlacliente($cdcliente, $cdassunto, $cdgrupfase, $idprazosla, $qtdiassla, $dtiniciosla)
    {
        $environment = InstanceProvider::getLibEnvironment();
        $nrorg = $environment->getCurrentOrganizationId();
        $currentOperator = $environment->getCurrentUserId();
        $dtAtual = new \DateTime();
        try {
            $dtiniciosla = empty($dtiniciosla) ? null : $dtAtual::createFromFormat('Y-m-d', $dtiniciosla);
            $qtdiassla = empty($qtdiassla) ? 0 : $qtdiassla / 8;
            $cdcomplexid = '02';
            $slacliente = $this->slaclienteFactory->create($cdcliente, $cdgrupfase, $cdassunto, $idprazosla, $qtdiassla,
                                                           $cdcomplexid, $nrorg, $currentOperator, $dtAtual, 'S',
                                                           $dtiniciosla);
            $this->entityManager->persist($slacliente);
            $this->entityManager->flush();
        } catch(Exception $e) {
            throw new Exception($e->getMessage());
        }
    }

    /**
     * @param $rows
     * @throws Exception
     */
    public function deleteSlacliente($rows)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            foreach ($rows as $row) {
                $filter = array(
                    'cdcliente' => $row['CDCLIENTE'],
                    'cdgrupfase' => $row['CDGRUPFASE'],
                    "cdassunto" => $row['CDASSUNTO'],
                    "cdcomplexid" => $row['CDCOMPLEXID']
                );
                /** @var \Teknisa\Model\Teknisa\Entities\Slacliente $slacliente */
                $slacliente = $this->slaclienteRepository->findOneBy($filter);
                $slaclienteAlert = InstanceProvider::getRepository()->getSlaclienteAlert()->findOneBy($filter);
                if (!empty($slaclienteAlert)) {
                    throw new Exception(Messages::ERR_NOT_DELETE_SLACLIENTE);
                }
                $this->entityManager->remove($slacliente);
            }
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch(Exception $e) {
            $connection->rollBack();
            throw new Exception($e->getMessage());
        }
    }

    public function validationsPermissisonDelete(string $cdcliente)
    {
        $slaclienteParamRepository = InstanceProvider::getRepository()->getSlaclienteParam();
        /** @var \Teknisa\Model\Teknisa\Entities\SlaclienteParam $slacliente */
        $slacliente = $slaclienteParamRepository->findOneBy(array('cdcliente' => $cdcliente));
        if ($slacliente->getIdParamSla() === 'P') {
            throw new \Exception(Messages::ERR_NOT_DELETE_PERMISSION_SLACLIENTE);
        }
    }
}
