/* Sistemas Integrados TECNISA
   Financeiro

Nome: 1045_RECSTP_INCLUI_INTRCOBRANCA.SQL

Descrio:
Stored procedure auxiliar para preenchimento da tabela INSTCOBRANCA.

P_CDBANCCNAB - Cdigo oficial do banco
P_CDINSTCOBR - Cdigo da intruo de cobrana
P_NMINSTCOBR - Descrio da intruo de cobrana

Para aplicar, executar e debugar
-- EXEC INCLUI_INSTCOBRANCA '341','26','NO RECEBER APS 40 DIAS DO VENCIMENTO';

Criao:
28/05/02 - Bruno Leonardo
*/

IF EXISTS (SELECT
              *
           FROM SYSOBJECTS O, SYSUSERS U
           WHERE
              O.NAME = 'INCLUI_INSTCOBRANCA' AND
              TYPE   = 'P'                   AND
              U.UID  = O.UID                 AND
              U.NAME = USER)

DROP PROCEDURE INCLUI_INSTCOBRANCA
GO

CREATE PROCEDURE INCLUI_INSTCOBRANCA
   @P_CDBANCCNAB  VARCHAR(03),
   @P_CDINSTCOBR  VARCHAR(02),
   @P_NMINSTCOBR  VARCHAR(100)

AS

   DECLARE @V_ACHOU   INTEGER;
   DECLARE @V_CDBANCO VARCHAR(03);

BEGIN
   SELECT
      @V_ACHOU = COUNT(*)
   FROM
      INSTCOBRANCA IC, BANCO BC
   WHERE
      (IC.CDBANCO = BC.CDBANCO) AND
      (BC.CDBANCCNAB = @P_CDBANCCNAB) AND
      (IC.CDINSTCOBR = @P_CDINSTCOBR);

   IF @V_ACHOU = 0 BEGIN

      SELECT
         @V_CDBANCO = MAX(CDBANCO)
      FROM
         BANCO
      WHERE
         (CDBANCCNAB = @P_CDBANCCNAB);

      IF NOT @V_CDBANCO IS NULL BEGIN
         INSERT INTO INSTCOBRANCA VALUES(@V_CDBANCO, @P_CDINSTCOBR, @P_NMINSTCOBR);
      END;
   END;
END;
GO

--ITAU 341

   EXEC INCLUI_INSTCOBRANCA '341','02','DEVOLVER APS 05 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','03','DEVOLVER APS 30 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','05','RECEBER CONFORME INSTRUES NO PRPRIO TTULO';
   EXEC INCLUI_INSTCOBRANCA '341','06','DEVOLVER APS 10 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','07','DEVOLVER APS 15 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','08','DEVOLVER APS 20 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','09','PROTESTAR';
   EXEC INCLUI_INSTCOBRANCA '341','10','NO PROTESTAR';
   EXEC INCLUI_INSTCOBRANCA '341','11','DEVOLVER APS 25 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','12','DEVOLVER APS 35 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','13','DEVOLVER APS 40 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','14','DEVOLVER APS 45 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','15','DEVOLVER APS 50 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','16','DEVOLVER APS 55 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','17','DEVOLVER APS 60 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','18','DEVOLVER APS 90 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','19','NO RECEBER APS 05 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','20','NO RECEBER APS 10 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','21','NO RECEBER APS 15 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','22','NO RECEBER APS 20 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','23','NO RECEBER APS 25 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','24','NO RECEBER APS 30 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','25','NO RECEBER APS 35 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','26','NO RECEBER APS 40 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','27','NO RECEBER APS 45 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','28','NO RECEBER APS 50 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','29','NO RECEBER APS 55 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','30','IMPORTNCIA DE DESCONTO POR DIA';
   EXEC INCLUI_INSTCOBRANCA '341','31','NO RECEBER APS 60 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','32','NO RECEBER APS 90 DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','36','PROTESTO URGENTE';
   EXEC INCLUI_INSTCOBRANCA '341','37','RECEBER AT O LTIMO DIA DO MS DE VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','38','CONCEDER DESCONTO MESMO APS VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','39','NO RECEBER APS O VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','40','CONCEDER DESCONTO CONFORME NOTA DE CRDITO';
   EXEC INCLUI_INSTCOBRANCA '341','43','SUJEITO A PROTESTO SE NO FOR PAGO NO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','44','IMPORTNCIA POR DIA DE ATRASO A PARTIR DE DDMMAA';
   EXEC INCLUI_INSTCOBRANCA '341','45','TEM DIA DA GRAA';
   EXEC INCLUI_INSTCOBRANCA '341','46','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','47','DISPENSAR JUROS COMISSO DE PERMANNCIA';
   EXEC INCLUI_INSTCOBRANCA '341','51','RECEBER SOMENTE COM A PARCELA ANTERIOR QUITADA';
   EXEC INCLUI_INSTCOBRANCA '341','52','FAVOR EFETUAR PGTO SOMENTE ATRAVS DESTA COBRANA BANCRIA';
   EXEC INCLUI_INSTCOBRANCA '341','53','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','54','APS VENCIMENTO PAGVEL SOMENTE NA EMPRESA';
   EXEC INCLUI_INSTCOBRANCA '341','56','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','57','SOMAR VALOR DO TTULO AO VALOR DO CAMPO MORA  MULTA CASO EXISTA';
   EXEC INCLUI_INSTCOBRANCA '341','58','DEVOLVER APS 365 DIAS DE VENCIDO';
   EXEC INCLUI_INSTCOBRANCA '341','59','COBRANA NEGOCIADA PAGVEL SOMENTE POR ESTE BLOQUETO NA REDE BANCRIA';
   EXEC INCLUI_INSTCOBRANCA '341','70','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','71','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','72','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','73','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','74','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','75','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','78','VALOR DA IDA ENGLOBA MULTA DE 10% PRO RATA';
   EXEC INCLUI_INSTCOBRANCA '341','79','COBRAR JUROS APS 15 DIAS DA EMISSO';
   EXEC INCLUI_INSTCOBRANCA '341','80','PAGAMENTO EM CHEQUE: SOMENTE RECEBER COM CHEQUE DE EMISSO DO SACADO';
   EXEC INCLUI_INSTCOBRANCA '341','81','PROTESTAR APS XX DIAS CORRIDOS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','82','PROTESTAR APS XX DIAS TEIS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','83','OPERAO REF A VENDOR';
   EXEC INCLUI_INSTCOBRANCA '341','84','APS VENCIMENTO CONSULTAR A AGNCIA CEDENTE';
   EXEC INCLUI_INSTCOBRANCA '341','86','ANTES DO VENCIMENTO OU APS 15 DIAS, PAGVEL SOMENTE EM NOSSA SEDE';
   EXEC INCLUI_INSTCOBRANCA '341','87','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','88','NO RECEBER ANTES DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','89','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','90','NO VENCIMENTO PAGVEL EM QUALQUER AGNCIA BANCRIA';
   EXEC INCLUI_INSTCOBRANCA '341','91','NO RECEBER APS XX DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','92','DEVOLVER APS XX DIAS DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '341','93','MENSAGENS NOS BLOQUETOS COM 30 POSIES';
   EXEC INCLUI_INSTCOBRANCA '341','94','MENSAGENS NOS BLOQUETOS COM 40 POSIES';
   EXEC INCLUI_INSTCOBRANCA '341','95','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','96','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','97','USO DO BANCO';
   EXEC INCLUI_INSTCOBRANCA '341','98','DUPLICATA FATURA N';

--BRADESCO 237

   EXEC INCLUI_INSTCOBRANCA '237','08','NO COBRAR JUROS DE MORA';
   EXEC INCLUI_INSTCOBRANCA '237','09','NO RECEBER APS O VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '237','08','NO COBRAR JUROS DE MORA';
   EXEC INCLUI_INSTCOBRANCA '237','09','NO RECEBER APS O VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '237','10','MULTA DE 10% APS O QUARTO DIA DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '237','11','NO RECEBER APS O 8% DIA DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '237','12','COBRAR ENCARGOS APS O 5 DIA DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '237','13','COBRAR ENCARGOS APOS O 10 DIA DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '237','14','COBRAR ENCARGOS APS O 15 DIA DO VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '237','15','CONCEDER DESCONTO MESMO SE PAGO APS O VENCIMENTO';

--UNIBANCO 409

   EXEC INCLUI_INSTCOBRANCA '409','01','NO RECEBER O PRINCIPAL SEM OS ENCARGOS DE MORA';
   EXEC INCLUI_INSTCOBRANCA '409','02','DEVOLVER SE NO PAGO AT 15 DIAS APS O VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '409','03','DEVOLVER SE NO PAGO AT 30 DIAS APS O VENCIMENTO';
   EXEC INCLUI_INSTCOBRANCA '409','04','DEVOLVER S A PEDIDO';
   EXEC INCLUI_INSTCOBRANCA '409','05','ACATAR AS INSTRUES CONTIDAS NO TTULO';
   EXEC INCLUI_INSTCOBRANCA '409','06','PROTESTAR';
   EXEC INCLUI_INSTCOBRANCA '409','07','NO PROTESTAR';
   EXEC INCLUI_INSTCOBRANCA '409','08','NO COBRAR JUROS DE MORA';

--BOSTON 479

   EXEC INCLUI_INSTCOBRANCA '479','00','S/ INSTR. NVEL TTULO';
   EXEC INCLUI_INSTCOBRANCA '479','01','PROT. APS "DD" DIAS';
   EXEC INCLUI_INSTCOBRANCA '479','02','NO PROTESTAR';
   EXEC INCLUI_INSTCOBRANCA '479','03','DEVOLVER APS "DD" DIAS';
