/* Sistemas Integrados TEKNISA
   Compras e Suprimentos

Nome: 8075_CMPSTPO_GERA_SOLICITACAO.sql

Descrio:
Stored procedure para gerao de solicitao de compras a partir dos registros
da tabela de trabalho TRABNECECOMP retornada pela stored procedure
NECESSIDADE_DE_COMPRAS.

P_CDFILILANC - Filial de lanamento (que gerou a solicitao).
P_NRNECECOMP - Nmero sequencial utilizado no clculo.
P_CDOPERLANC - Operador que lanou a solicitao de compras

Criao:
22/01/1999 Leonardo Monteiro.
*/

IF EXISTS (SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'GERA_SOLICITACAO'  and
            type = 'P'                  and
            u.uid = o.uid               and
            u.name = USER)
   DROP PROCEDURE GERA_SOLICITACAO;
GO

CREATE PROCEDURE GERA_SOLICITACAO(
   @P_CDFILILANC VARCHAR(4),
   @P_NRNECECOMP VARCHAR(10),
   @P_CDOPERLANC VARCHAR(12),
   @P_EXTRA      VARCHAR(1) = 'N',
   @P_MSGEXTRA   VARCHAR(50) = NULL) AS

DECLARE @V_PRXSOLICMP   VARCHAR(20);
DECLARE @V_IDUTAPROVSOL VARCHAR(1);
DECLARE @V_CDFILCENT    VARCHAR(4);
DECLARE @V_IDSOLIAPROV  VARCHAR(1);
DECLARE @VCUR_FILIAL VARCHAR(4);
DECLARE @VSOLICOMP VARCHAR(200);
DECLARE @V_CDFILCENTPME VARCHAR(4);
DECLARE @V_IDFILCENTPME VARCHAR(1);
DECLARE @V_CDFILNECECMP VARCHAR(4);
DECLARE @V_CDPRODNECCMP VARCHAR(10);
DECLARE @V_CDPRODESTO VARCHAR(10);
DECLARE @V_DTENTRNECCMP DATETIME;
DECLARE @V_IDUTILDTPROC VARCHAR(1);
DECLARE @V_DTPROCESS    DATETIME;
DECLARE @V_CDFILSOLI  VARCHAR(4);
DECLARE @V_CDFILLAN   VARCHAR(4);
DECLARE @V_NRSOLICMP  VARCHAR(10);
DECLARE @V_CDPRODUTO  VARCHAR(10);
DECLARE @V_IDUTILCONSSOLIOL VARCHAR(1);
DECLARE @V_DTPROGENTR DATETIME;
DECLARE @V_IDPERALTSAPR VARCHAR(1);
DECLARE @V_CDFILITIPOCARD VARCHAR(4);

BEGIN
   SELECT
      @V_IDUTILDTPROC = MAX(IDUTILDTPROC)
   FROM
      PARAMGERAL;

   SELECT
      @V_IDUTILCONSSOLIOL = ISNULL(MAX(IDUTILCONSSOLIOL), 'N')
   FROM
      PARASUPRI
   WHERE
      PARASUPRI.CDFILIAL = (SELECT ISNULL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)
                            FROM   FILIAL
                            WHERE  FILIAL.CDFILIAL = @P_CDFILILANC
                            );
   DECLARE CUR_FILIAL CURSOR FOR
      (SELECT DISTINCT
          TRABNECECOMP.CDFILNECECMP
       FROM
          TRABNECECOMP
       WHERE
          TRABNECECOMP.NRNECECOMP = @P_NRNECECOMP)
   OPEN CUR_FILIAL;
   FETCH NEXT FROM CUR_FILIAL INTO @VCUR_FILIAL;
   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      SET @V_IDUTAPROVSOL = '';
      SET @V_PRXSOLICMP   = '';
      SET @V_CDFILCENT    = '';
      SET @V_IDSOLIAPROV  = '';
      SET @V_IDPERALTSAPR = '';
      SET @VSOLICOMP      = 'SOLICOMP'+@VCUR_FILIAL+@P_CDFILILANC;
      EXEC NOVO_CODIGO @VSOLICOMP, @P_SEQUENCIAL = @V_PRXSOLICMP OUTPUT, @P_QTDE = 1;
      SET @V_PRXSOLICMP = SUBSTRING(@V_PRXSOLICMP,11,10);

      SELECT
         @V_IDUTAPROVSOL = MAX(IDUTAPROVSOL),
         @V_IDPERALTSAPR = MAX(SUBSTRING(ISNULL(IDPERALTSAPR, 'NNN'),3,1))
      FROM
         PARCMPSUP
      WHERE
         CDFILIAL = @VCUR_FILIAL;

      IF @V_IDUTAPROVSOL = 'S' BEGIN
         IF @V_IDPERALTSAPR = 'S' BEGIN
            SET @V_IDSOLIAPROV = 'S';
         END ELSE BEGIN
            SET @V_IDSOLIAPROV = 'N';
         END;
      END ELSE BEGIN
         SET @V_IDSOLIAPROV = 'S';
      END;

      SELECT
         @V_CDFILCENT    = MAX(ISNULL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)),
         @V_CDFILCENTPME = MAX(CDFILCENTPME),
         @V_IDFILCENTPME = MAX(IDFILCENTPME)
      FROM
         FILIAL
      WHERE
         FILIAL.CDFILIAL = @VCUR_FILIAL;

      INSERT INTO SOLICITA
        (CDFILSOLI,
         CDFILLANCA,
         NRSOLICMP,
         DTSOLCMP,
         IDTIPLAN,
         CDFILCENT,
         CDOPERLANC,
         NRNECECMPSOL,
         IDSOLEXTRA)
      VALUES
        (@VCUR_FILIAL,
         @P_CDFILILANC,
         @V_PRXSOLICMP,
         CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103),
         'A',
         @V_CDFILCENT,
         @P_CDOPERLANC,
         @P_NRNECECOMP,
         @P_EXTRA);

      IF @P_EXTRA = 'S' BEGIN
         INSERT INTO MOVSOLEXT
            (CDFILSOLEXT,
             CDFILLANCEXT,
             NRSOLIMOVEXT,
             DTMOVSOLEXT,
             CDOPERADOR,
             DSJUSTSOLEXT)
         VALUES
            (@VCUR_FILIAL,
             @P_CDFILILANC,
             @V_PRXSOLICMP,
             GETDATE(),
             @P_CDOPERLANC,
             @P_MSGEXTRA);
      END;

      IF (@V_CDFILCENTPME IS NOT NULL) AND (ISNULL(@V_IDFILCENTPME, 'N') = 'N') BEGIN
         SELECT
            @V_CDFILITIPOCARD = MAX(CDFILIAL)
         FROM
            FILITIPOCARD
         WHERE
            CDFILCENTPME = @V_CDFILCENTPME AND
            CDFILIAL     = @VCUR_FILIAL;

         IF (@V_CDFILITIPOCARD IS NOT NULL) AND (@V_IDUTAPROVSOL = 'S') BEGIN
            SET @V_IDSOLIAPROV = 'N';
         END;
      END;

      IF @V_IDUTILCONSSOLIOL = 'S' BEGIN
         INSERT INTO ITEMSOLI
           (CDFILSOLI,
            CDFILLAN,
            NRSOLICMP,
            CDPRODUTO,
            DTPROGENTR,
            DTUTILIZA,
            QTSOLIPROD,
            IDSOLIAPROV,
            CDFILIOPELOG
            )
         SELECT
            @VCUR_FILIAL,
            @P_CDFILILANC,
            @V_PRXSOLICMP,
            CDPRODNECCMP,
            DTENTRNECCMP,
            DTPRIUTIPROD,
            QTPRCNVCOMPR,
            @V_IDSOLIAPROV,
            (SELECT
                MAX(P.CDFILIOPELOG)
             FROM
                PARAMPRODSUP P
             WHERE
                P.CDFILIAL     = @VCUR_FILIAL AND
                P.CDPRODUTO    = CDPRODNECCMP AND
                P.CDFILIOPELOG IS NOT NULL
             )
         FROM
            TRABNECECOMP
         WHERE
            (NRNECECOMP   = @P_NRNECECOMP ) AND
            (IDMARCNECCMP = 'S'           ) AND
            (CDFILNECECMP = @VCUR_FILIAL);
      END
      ELSE BEGIN
         INSERT INTO ITEMSOLI
           (CDFILSOLI,
            CDFILLAN,
            NRSOLICMP,
            CDPRODUTO,
            DTPROGENTR,
            DTUTILIZA,
            QTSOLIPROD,
            IDSOLIAPROV
            )
         SELECT
            @VCUR_FILIAL,
            @P_CDFILILANC,
            @V_PRXSOLICMP,
            CDPRODNECCMP,
            DTENTRNECCMP,
            DTPRIUTIPROD,
            QTPRCNVCOMPR,
            @V_IDSOLIAPROV
         FROM
            TRABNECECOMP
         WHERE
            (NRNECECOMP   = @P_NRNECECOMP ) AND
            (IDMARCNECCMP = 'S'           ) AND
            (CDFILNECECMP = @VCUR_FILIAL);
      END;

      IF @V_IDUTILDTPROC <> 'S'
      BEGIN
         SET @V_DTPROCESS = CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103);
      END
      ELSE
      BEGIN
         SELECT
            @V_DTPROCESS = ISNULL(MAX(DTPROCESSA),CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103))
         FROM
            PARAMFILIAL
         WHERE
            (CDFILIAL = @VCUR_FILIAL);
      END;

      DECLARE C_FILINECE CURSOR FOR
         (SELECT
             IT.CDFILSOLI,
             IT.CDFILLAN,
             IT.NRSOLICMP,
             IT.CDPRODUTO,
             IT.DTPROGENTR
          FROM
             PRODUTO PR,
             ITEMSOLI IT,
             FILIAL,
             FILINECE FI,
             LOGNECECOMP LOG
          WHERE
             LOG.CDFILIAL             = @VCUR_FILIAL                    AND
             LOG.NRNECECMPLOG         = @P_NRNECECOMP                   AND
             FI.CDFILIAL              = LOG.CDFILIAL                    AND
             FI.IDSOLIPREV           <> 'P'                             AND
             FI.CDFILIAL              = FILIAL.CDFILIAL                 AND
             ISNULL(FILIAL.IDFILCENTPME, 'N') = 'N'                        AND
             FI.CDFILINECCMP          = IT.CDFILSOLI                    AND
             IT.CDPRODUTO             = PR.CDPRODUTO                    AND
             IT.DTPROGENTR           >= @V_DTPROCESS                    AND
             PR.CDARVPROD BETWEEN LOG.CDPRODINICNC AND LOG.CDPRODFINANC
          )
      OPEN C_FILINECE;
      FETCH NEXT FROM C_FILINECE INTO @V_CDFILSOLI,
                                      @V_CDFILLAN,
                                      @V_NRSOLICMP,
                                      @V_CDPRODUTO,
                                      @V_DTPROGENTR;
      WHILE (@@FETCH_STATUS = 0)
      BEGIN
         UPDATE ITEMSOLI
         SET IDITMCONS = 'S'
         WHERE
            ITEMSOLI.CDFILSOLI  = @V_CDFILSOLI  AND
            ITEMSOLI.CDFILLAN   = @V_CDFILLAN   AND
            ITEMSOLI.NRSOLICMP  = @V_NRSOLICMP  AND
            ITEMSOLI.CDPRODUTO  = @V_CDPRODUTO  AND
            ITEMSOLI.DTPROGENTR = @V_DTPROGENTR;
         FETCH NEXT FROM C_FILINECE INTO @V_CDFILSOLI,
                                         @V_CDFILLAN,
                                         @V_NRSOLICMP,
                                         @V_CDPRODUTO,
                                         @V_DTPROGENTR;
      END;
      CLOSE C_FILINECE;
      DEALLOCATE C_FILINECE;

      IF (@V_CDFILCENTPME IS NULL) AND (@V_IDFILCENTPME = 'S') BEGIN
         DECLARE CUR_NECECOMP CURSOR FOR
           SELECT
               TRABNECECOMP.CDFILNECECMP,
               TRABNECECOMP.CDPRODNECCMP,
               PRODUTO.CDPRODESTO,
               TRABNECECOMP.DTENTRNECCMP
            FROM
               PRODUTO,
               TRABNECECOMP
            WHERE
               PRODUTO.CDPRODUTO = TRABNECECOMP.CDPRODNECCMP AND
               NRNECECOMP   = @P_NRNECECOMP                   AND
               IDMARCNECCMP = 'S'                            AND
               CDFILNECECMP = @VCUR_FILIAL
            ORDER BY
               DTENTRNECCMP DESC
         OPEN CUR_NECECOMP;
         FETCH NEXT FROM CUR_NECECOMP INTO @V_CDFILNECECMP,
                                           @V_CDPRODNECCMP,
                                           @V_CDPRODESTO,
                                           @V_DTENTRNECCMP;
         WHILE (@@FETCH_STATUS = 0)
         BEGIN
            UPDATE
               ITEMSOLI
            SET
               CDFILSOLCONS = @V_CDFILNECECMP,
               CDFILLANCONS = @P_CDFILILANC,
               NRSOLCMPCONS = @V_PRXSOLICMP,
               CDPRODUTCONS = @V_CDPRODNECCMP,
               DTPROGENCONS = @V_DTENTRNECCMP,
               IDITMCONS    = 'S'
            WHERE
               CONVERT(VARCHAR, DTPROGENTR, 103)+CDPRODUTO+CDFILSOLI IN (SELECT
                                                                            CONVERT(VARCHAR,IT.DTPROGENTR,103)+IT.CDPRODUTO+IT.CDFILSOLI
                                                                         FROM
                                                                            PRODUTO,
                                                                            FILIAL,
                                                                            ITEMSOLI IT
                                                                         WHERE
                                                                            IT.CDFILSOLCONS IS NULL                          AND
                                                                            ISNULL(IT.IDITMCONS, 'N')  = 'N'                    AND
                                                                            ISNULL(IT.IDSOLIAPROV, 'S')= 'S'                    AND
                                                                            IT.CDPRODUTO         = PRODUTO.CDPRODUTO         AND
                                                                            PRODUTO.CDPRODESTO   = @V_CDPRODESTO   AND
                                                                            FILIAL.CDFILIAL      = IT.CDFILSOLI              AND
                                                                            FILIAL.CDFILCENTPME  = @V_CDFILNECECMP AND
                                                                            IT.DTPROGENTR >= @V_DTENTRNECCMP AND
                                                                            NOT EXISTS  (SELECT
                                                                                            CMPCENTENDIR.CDFILIAL,
                                                                                            CMPCENTENDIR.CDPRODUTO
                                                                                         FROM
                                                                                            CMPCENTENDIR,
                                                                                            PRODUTO PI,
                                                                                            PRODUTO PC
                                                                                         WHERE
                                                                                            CMPCENTENDIR.CDFILIAL  = IT.CDFILSOLI AND
                                                                                            CMPCENTENDIR.CDPRODUTO = PC.CDPRODUTO AND
                                                                                            IT.CDPRODUTO           = PI.CDPRODUTO AND
                                                                                            PI.CDARVPROD LIKE PC.CDARVPROD+'%'));
         FETCH NEXT FROM CUR_NECECOMP INTO @V_CDFILNECECMP,
                                           @V_CDPRODNECCMP,
                                           @V_CDPRODESTO,
                                           @V_DTENTRNECCMP;
         END;
            CLOSE CUR_NECECOMP;
            DEALLOCATE CUR_NECECOMP;
      END;

      DELETE FROM TRABNECECOMP
      WHERE
         (NRNECECOMP   = @P_NRNECECOMP)  AND
         (IDMARCNECCMP = 'S'          )  AND
         (CDFILNECECMP = @VCUR_FILIAL);
      FETCH NEXT FROM CUR_FILIAL INTO @VCUR_FILIAL;
   END;
   CLOSE CUR_FILIAL;
   DEALLOCATE CUR_FILIAL;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'GERA_SOLICITACAO','5.05.022';
GO

