import { OMInstance } from '@/services/OMInstance';
import ApiSlacliente from '@/services/api/Slacliente';
import ApiSlaclienteParam from '@/services/api/SlaclienteParam';
import ApiSlaclienteAlert from '@/services/api/SlaclienteAlert';
import ApiClassification from '@/services/api/Classification';
import { IDatasource, IDictionary, IEventParam, Metadata } from '@zeedhi/core';
import { TekAgGrid } from '@zeedhi/teknisa-components-common';
import { Modal, ModalService, Select, TextInput } from '@zeedhi/common';
import { OmDialogService } from '@/services/OmDialogService';
import Messages from '@/services/Messages';
import ScreenService from '@/services/ScreenService';
import Store from '../store/index';

export class SlaclienteController {
	public timeout = 0;

	/** **************************************************************************
	 *************************** Section Grid Cliente ****************************
	 *************************************************************************** */

	/** properties */
	private cdclienteTeknisa = 'J26269316000177';

	private clienteGridCurrentRow: any = {};

	public editedRowsClienteGrid: Array<object> = [];

	public editedRowsControlClienteGrid: Array<any> = [];

	public gridApi: any;

	private dataSourceIdParamSla = [
		{
			id: 'P',
			name: 'SLA Padrão Teknisa',
		},
		{
			id: 'C',
			name: 'SLA Cliente',
		},
	];

	private dataSourceIdControlaSla = [
		{
			id: 'S',
			name: 'Sim (cliente possui contrato)',
		},
		{
			id: 'I',
			name: 'Sim (apenas controle Teknisa)',
		},
		{
			id: 'N',
			name: 'Não',
		},
	];

	/** Accessors */

	get hasEditedRowsClienteGrid() {
		return !!this.editedRowsClienteGrid.length;
	}

	get hasNoEditedRowsClienteGrid() {
		return !this.editedRowsClienteGrid.length;
	}

	get nomeFatasia() {
		return this.clienteGridCurrentRow.NMFANTCLIE || '';
	}

	public editableColIdParamSla(param: any) {
		return param.data.ID_CONTROLA_SLA !== 'N';
	}

	/** Actions  */

	public async reloadClienteGrid(): Promise<any> {
		const { datasource } = OMInstance.getInstanceTekAgGrid('clienteGrid');
		let { value } = OMInstance.getInstanceZdTextInput('searchFieldClienteGrid');
		value = value ? value.toString() : '';
		this.setDatasourceSearchInAllByGrid(value, datasource, 'clienteGrid');
		this.showLoadingClienteGrid();
		try {
			const response = await datasource.get();
			this.hideLoadingClienteGrid();
			this.clienteGridCurrentRow = {};
			this.slaclienteGridCurrentRow = {};
			return response;
		} catch (error) {
			return false;
		}
	}

	public async saveClienteGrid() {
		try {
			this.showLoadingClienteGrid();
			await ApiSlaclienteParam.updateSlaclienteParam(this.editedRowsClienteGrid);
			OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
			this.resetEditRows();
		} catch (error) {
			OmDialogService.showErrorBackend(error);
		} finally {
			this.hideLoadingClienteGrid();
		}
	}

	public async cancelClienteGrid() {
		await this.reloadClienteGrid();
		this.resetEditRows();
	}

	/** Events  */

	public onMounted({ component }: IEventParam<TekAgGrid>) {
		setTimeout(() => {
			this.gridApi = component.gridComponent.gridOptions.api;
		}, 500);
	}

	public async onRowClickedClienteGrid(events: any) {
		try {
			const currentCdcliente = events.data.CDCLIENTE;
			const paramSlaGrid = OMInstance.getInstanceTekAgGrid('slaclienteGrid');
			const { datasource } = paramSlaGrid;
			datasource.filter.CDCLIENTE = currentCdcliente;
			await Store.commit('Cliente/SET_CLIENTE', events.data);
			await Store.dispatch('Cliente/setSlacliente', {});
			this.setClienteGridCurrentRow(events.data);
			this.slaclienteGridCurrentRow = {};
			this.resetEditRowsSlaclienteGrid();
			if (events.data.ID_CONTROLA_SLA !== 'N') {
				if (events.data.ID_PARAM_SLA === 'P') {
					datasource.filter.CDCLIENTE = this.cdclienteTeknisa;
				}
				this.showLoadingSlaclienteGrid();
				this.slaclienteGridDataBackup = await datasource.get();
				this.hideLoadingSlaclienteGrid();
			}
		} catch (e) {
			OmDialogService.showErrorBackend(e);
			this.hideLoadingSlaclienteGrid();
		}
	}

	public onCellValueChangedClienteGrid(event: any) {
		const cdcliente = event.data.CDCLIENTE;
		const rowNode = event.api.getRowNode(event.rowIndex);

		if (event.data.ID_CONTROLA_SLA === 'N') {
			event.data.LABEL_PARAM_SLA = '';
			event.data.ID_PARAM_SLA = '';
			rowNode.setData(event.data);
		}

		if (!this.editedRowsControlClienteGrid.includes(cdcliente)) {
			this.editedRowsControlClienteGrid.push(cdcliente);
			this.editedRowsClienteGrid.push(event.data);
		} else {
			const index = this.editedRowsControlClienteGrid.indexOf(cdcliente);
			this.editedRowsClienteGrid[index] = event.data;
		}
	}

	/** methods */

	public resetEditRows() {
		this.editedRowsClienteGrid = [];
		this.editedRowsControlClienteGrid = [];
	}

	public selectFormatterIdParamSla(params: any) {
		const foundRow = this.dataSourceIdParamSla.find((row: any) => row.id === params.value);
		return !foundRow ? params.value : foundRow.name;
	}

	public selectFormatterIdControlaSla(params: any) {
		const foundRow = this.dataSourceIdControlaSla.find((row: any) => row.id === params.value);
		return !foundRow ? params.value : foundRow.name;
	}

	public setClienteGridCurrentRow(row: object) {
		this.clienteGridCurrentRow = row;
	}

	public showLoadingClienteGrid() {
		const { gridComponent } = OMInstance.getInstanceTekAgGrid('clienteGrid');
		gridComponent.gridOptions.api.showLoadingOverlay();
	}

	public hideLoadingClienteGrid() {
		const { gridComponent } = OMInstance.getInstanceTekAgGrid('clienteGrid');
		gridComponent.gridOptions.api.hideOverlay();
	}

	public prependIconClickSearchFieldClienteGrid(params: IEventParam<any>) {
		console.log(params);
	}

	public async appendIconClickSearchFieldClienteGrid() {
		await this.reloadClienteGrid();
	}

	public async keyupSearchFieldClienteGrid() {
		clearTimeout(this.timeout);
		this.timeout = setTimeout(() => {
			this.reloadClienteGrid();
		}, 750);
	}

	/** *************************************************************************************
	 *************************** Section Grid Sla Cliente **********************************
	 ************************************************************************************** */

	/** properties */

	public slaclienteGridCurrentRow: any = {};

	public editedRowsSlaclienteGrid: Array<object> = [];

	public editedRowsControlSlaclienteGrid: Array<any> = [];

	public cachedData?: IDictionary[];

	private dataSourceCdgrupofase = [
		{
			id: '13',
			name: 'Especificação',
		},
		{
			id: '30',
			name: 'Implementação',
		},
	];

	private disableDeleteActionSlaclienteGrid = true;

	private visibilityCheckboxSelectionSlaclienteGrid = true;

	public gridOptionsSlaclienteGrid = {};

	public visibilityHeaderCheckboxSelectionSlaclienteGrid = true;

	public slaclienteGridDataBackup = [];

	/** Accessors */

	get heightSlaclienteGrid(): string {
		if (this.clienteGridCurrentRow.CDCLIENTE && this.clienteGridCurrentRow.ID_CONTROLA_SLA !== 'N') {
			return '240px';
		}
		return '0px';
	}

	get slaclienteGridVisibility(): boolean {
		return this.clienteGridCurrentRow.CDCLIENTE && this.clienteGridCurrentRow.ID_CONTROLA_SLA !== 'N';
	}

	get slaclienteActionsGridVisibility() {
		let visibility = true;
		try {
			const { datasource } = OMInstance.getInstanceTekAgGrid('slaclienteGrid');
			visibility = !!datasource.data.length;
		} catch (e) {
			visibility = false;
		}
		return visibility;
	}

	get hasEditedRowsSlaclienteGrid() {
		const currentCliente = Store.getters['Cliente/getCliente'];
		return !!this.editedRowsSlaclienteGrid.length || currentCliente.ID_PARAM_SLA === 'P';
	}

	get hasNoEditedRowsSlaclienteGrid() {
		return !this.editedRowsSlaclienteGrid.length;
	}

	get isVisibleCheckboxSelectionSlaclienteGrid() {
		return this.visibilityCheckboxSelectionSlaclienteGrid;
	}

	get isDisableDeleteActionSlaclienteGrid() {
		const instances = Metadata.getInstances('slaclienteGrid');
		if (!instances[0]) {
			return false;
		}
		return this.disableDeleteActionSlaclienteGrid;
	}

	get descriptionOrder() {
		const slacliente = this.slaclienteGridCurrentRow;
		if (Object.keys(slacliente).length) {
			return `${slacliente.CDASSUNTO_LABEL} | ${slacliente.CDGRUPFASE_LABEL} | ${slacliente.IDPRAZOSLA_LABEL}`;
		}
		return '';
	}

	public isEditableColIdPrazoId(param: any) {
		return param.data.CDGRUPFASE !== '13' && this.clienteGridCurrentRow.ID_PARAM_SLA !== 'P';
	}

	public isEditableColQtdiasla(param: any) {
		return param.data.IDPRAZOSLA === '1' && this.clienteGridCurrentRow.ID_PARAM_SLA !== 'P';
	}

	public isEditableColDtiniciosla() {
		return this.clienteGridCurrentRow.ID_PARAM_SLA !== 'P';
	}

	public get getVisibiliteHeaderCheckboxSelection() {
		return this.visibilityHeaderCheckboxSelectionSlaclienteGrid;
	}

	/** Actions  */

	public async reloadSlaclienteGrid() {
		const { datasource } = OMInstance.getInstanceTekAgGrid('slaclienteGrid');
		try {
			this.showLoadingSlaclienteGrid();
			datasource.filter.CDCLIENTE = Store.getters['Cliente/getCdcliente'];
			await datasource.get();
			this.disableDeleteActionSlaclienteGrid = true;
		} catch (e) {
			OmDialogService.showErrorBackend(e);
		} finally {
			this.hideLoadingSlaclienteGrid();
		}
	}

	public async saveSlaclienteGrid() {
		try {
			this.showLoadingSlaclienteGrid();
			await ApiSlacliente.updateSlacliente(this.editedRowsSlaclienteGrid);
			this.resetEditRowsSlaclienteGrid();
			OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
		} catch (e) {
			OmDialogService.showErrorBackend(e);
		} finally {
			this.hideLoadingSlaclienteGrid();
		}
	}

	public async cancelSlaclienteGrid() {
		await this.reloadSlaclienteGrid();
		this.resetEditRowsSlaclienteGrid();
	}

	public async deleteSlaclienteGrid() {
		const { gridComponent } = OMInstance.getInstanceTekAgGrid('slaclienteGrid');
		const { api: gridApi } = gridComponent.gridOptions;
		const rowsNode = gridApi.getSelectedNodes();
		const rows = rowsNode.map((row: any) => {
			row.data.CDCLIENTE_SLACLIENTE_PARAM = this.clienteGridCurrentRow.CDCLIENTE;
			return row.data;
		});
		const params = { data: rows };
		this.showLoadingSlaclienteGrid();
		try {
			await ApiSlacliente.deleteSlacliente(params);
			gridApi.deselectAll();
			OmDialogService.showSuccess(Messages.SUCCESS_DELETE_DEAFULT);
			await this.reloadSlaclienteGrid();
		} catch (e) {
			OmDialogService.showErrorBackend(e);
		} finally {
			this.hideLoadingSlaclienteGrid();
		}
	}

	/** Events  */

	public onMountedSlacliente({ component }: IEventParam<TekAgGrid>) {
		setTimeout(() => {
			this.gridOptionsSlaclienteGrid = component.gridComponent.gridOptions;
		}, 500);
	}

	public onCellValueChangedSlaclienteGrid(event: any) {
		const { CDCLIENTE, CDGRUPFASE, CDASSUNTO, CDCOMPLEXID } = event.data;
		const id = CDCLIENTE + CDGRUPFASE + CDASSUNTO + CDCOMPLEXID;
		const rowNode = event.api.getRowNode(event.rowIndex);

		this.validationQtdiassla(event, rowNode);

		if (!this.editedRowsControlSlaclienteGrid.includes(id)) {
			this.editedRowsControlSlaclienteGrid.push(id);
			this.editedRowsSlaclienteGrid.push(event.data);
		} else {
			const index = this.editedRowsControlSlaclienteGrid.indexOf(id);
			this.editedRowsSlaclienteGrid[index] = event.data;
		}
	}

	public async onRowClickedSlaclienteGrid(param: any) {
		await Store.dispatch('Cliente/setSlacliente', param.data);
		this.slaclienteGridCurrentRow = param.data;
		const slaParamAlertGrid = OMInstance.getInstanceTekAgGrid('slaclienteAlertGrid');
		const { datasource } = slaParamAlertGrid;
		datasource.filter.CDCLIENTE = param.data.CDCLIENTE;
		datasource.filter.CDGRUPFASE = param.data.CDGRUPFASE;
		datasource.filter.CDASSUNTO = param.data.CDASSUNTO;
		datasource.filter.CDCOMPLEXID = param.data.CDCOMPLEXID;
		try {
			this.showLoadingSlaclienteAlertGrid();
			this.slaclienteAlertGridDataBackup = await datasource.get();
			this.resetEditRowsSlaclienteAlertGrid();
		} catch (e) {
			OmDialogService.showErrorBackend(e);
		} finally {
			this.hideLoadingSlaclienteAlertGrid();
		}
	}

	public onRowSelectedSlaclienteGrid(params: any) {
		this.disableDeleteActionSlaclienteGrid = !params.api.getSelectedNodes().length || this.clienteGridCurrentRow.ID_PARAM_SLA === 'P';
	}

	public onFilterChangedSlaclienteGrid(params: any) {
		console.log(params);
	}

	/** methods */

	public resetEditRowsSlaclienteGrid() {
		this.editedRowsSlaclienteGrid = [];
		this.editedRowsControlSlaclienteGrid = [];
		this.disableDeleteActionSlaclienteGrid = true;
	}

	public showLoadingSlaclienteGrid() {
		const { gridComponent } = OMInstance.getInstanceTekAgGrid('slaclienteGrid');
		gridComponent.gridOptions.api.showLoadingOverlay();
	}

	public hideLoadingSlaclienteGrid() {
		const { gridComponent } = OMInstance.getInstanceTekAgGrid('slaclienteGrid');
		gridComponent.gridOptions.api.hideOverlay();
	}

	public async selectFormatterCdassunto(params: any) {
		this.showLoadingSlaclienteGrid();
		if (!this.cachedData) {
			const { data: { data } } = await ApiClassification.findClassification({ params: { ONLY_SUBJECT: true } });
			this.cachedData = data;
		}

		if (!this.cachedData) {
			this.hideLoadingSlaclienteGrid();
			return params.value;
		}

		const foundRow = this.cachedData.find((row: any) => row.CDASSUNTO === params.value);
		if (!foundRow) {
			this.hideLoadingSlaclienteGrid();
			return params.value;
		}

		this.hideLoadingSlaclienteGrid();
		return foundRow.LABEL;
	}

	public selectFormatterCdgrupofase(params: any) {
		const { dataValue } = params.colDef.cellEditorParams.componentProps;
		const foundRow = this.dataSourceCdgrupofase.find((row: any) => row[dataValue] === params.value);
		return !foundRow ? params.value : foundRow.name;
	}

	public validationQtdiassla(event: any, rowNode: any) {
		if (event.data.IDPRAZOSLA === '2') {
			event.data.QTDIASSLA = '';
			rowNode.setData(event.data);
		}
	}

	public dateComparator(filterDate: Date, cellDate: string) {
		const grid = OMInstance.getInstanceTekAgGrid('slaclienteGrid');
		const cellDateArray = cellDate.split('/');
		const cellDateIverted = `${cellDateArray[1]}/${cellDateArray[0]}/${cellDateArray[2]}`;
		const newCellDate = new Date(cellDateIverted);
		return grid.dateComparator(filterDate, newCellDate);
	}

	public keyupSearchFieldSlaclienteGrid({ component }: IEventParam<any>) {
		clearTimeout(this.timeout);
		this.timeout = setTimeout(() => {
			const { datasource } = OMInstance.getInstanceTekAgGrid('slaclienteGrid');
			const { value } = component;
			if (value) {
				datasource.data = datasource.data.filter((row: any) => row.CDCLIENTE.toLowerCase().includes(value.toLowerCase())
					|| row.CDASSUNTO_LABEL.toLowerCase().includes(value.toLowerCase())
					|| row.CDGRUPFASE_LABEL.toLowerCase().includes(value.toLowerCase())
					|| row.IDPRAZOSLA_LABEL.toLowerCase().includes(value.toLowerCase()));
			} else {
				datasource.data = this.slaclienteGridDataBackup;
			}
		}, 750);
	}

	/** End Section */

	/** *************************************************************************************
	 *************************** Section Add Modal SLA Cliente ******************************
	 ************************************************************************************** */

	/** Modal Methods */

	private addSlaclienteModal!: Modal;

	public async createAddSlaclienteModal() {
		this.addSlaclienteModal = await ModalService.createFromJson('parametrizacaoSlaCliente/modal/addSlacliente', true);
	}

	public destroyAddSlaclienteModal() {
		this.addSlaclienteModal.destroy();
	}

	public openAddSlaclienteModal() {
		this.addSlaclienteModal.show();
	}

	public closeAddSlaclienteModal() {
		this.addSlaclienteModal.hide();
	}

	/** Form|Inputs Methods */

	public async saveAddSlaclienteModal() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteModalForm');
		const { CDCLIENTE } = Store.getters['Cliente/getCliente'];
		const row = { ...form.value, CDCLIENTE };
		if (form.validate()) {
			ScreenService.showLoader();
			try {
				await ApiSlacliente.saveSlacliente(row);
				this.closeAddSlaclienteModal();
				ScreenService.hideLoader();
				OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
				await this.reloadSlaclienteGrid();
			} catch (e) {
				ScreenService.hideLoader();
				OmDialogService.showErrorBackend(e);
			}
		}
	}

	public async cdassuntoOnChangeAddSlaclienteModalForm() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteModalForm');
		const cdassunto = form.getChildInstance<Select>('CDASSUNTO');
		if (this.validationOfValuesAlreadyRegisteredInCdassuntoAndCdgrupfase(form.value)) {
			await cdassunto.setValue('');
			OmDialogService.showError(Messages.ERROR_EQUAL_SLA_PARAMETERS);
		}
	}

	public async cdgrupfaseOnChangeAddSlaclienteModalForm() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteModalForm');
		const idprazosla = form.getChildInstance<Select>('IDPRAZOSLA');
		const cdgrupfase = form.getChildInstance<Select>('CDGRUPFASE');

		if (this.validationOfValuesAlreadyRegisteredInCdassuntoAndCdgrupfase(form.value)) {
			await cdgrupfase.setValue('');
			OmDialogService.showError(Messages.ERROR_EQUAL_SLA_PARAMETERS);
			return;
		}

		if (form.value.CDGRUPFASE === '13') {
			await idprazosla.setValue('1');
			idprazosla.readonly = true;
		} else {
			idprazosla.readonly = false;
		}
	}

	private validationOfValuesAlreadyRegisteredInCdassuntoAndCdgrupfase(formValues: any) {
		const { datasource: datasourceGrid } = OMInstance.getInstanceTekAgGrid('slaclienteGrid');
		return datasourceGrid.data.find((row) => row.CDASSUNTO === formValues.CDASSUNTO && row.CDGRUPFASE === formValues.CDGRUPFASE);
	}

	public async idprazoslaOnChangeAddSlaclienteModalForm() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteModalForm');
		const qtdiassla = form.getChildInstance<TextInput>('QTDIASSLA');
		if (form.value.IDPRAZOSLA !== '1') {
			qtdiassla.value = '';
			qtdiassla.readonly = true;
		} else {
			qtdiassla.readonly = false;
			qtdiassla.validations.required = {};
		}
	}

	/** *************************************************************************************
	 *************************** Section Grid Sla Cliente Alert ****************************
	 ************************************************************************************** */

	/** properties */

	public cachedDataStructureTypeId?: IDictionary[];

	public editedRowsSlaclienteAlertGrid: Array<object> = [];

	public editedRowsControlSlaclienteAlertGrid: Array<any> = [];

	private disableDeleteActionSlaclienteAlertGrid = true;

	private slaclienteAlertGridDataBackup = [];

	/** Accessors */

	get heightSlaclienteAlertGrid() {
		const currentSlacliente = Store.getters['Cliente/getSlacliente'];
		if (currentSlacliente.CDCLIENTE && this.clienteGridCurrentRow.ID_CONTROLA_SLA !== 'N') {
			return '240px';
		}
		return '0px';
	}

	get slaclienteAlertGridVisibility() {
		const currentSlacliente = Store.getters['Cliente/getSlacliente'];
		return !!currentSlacliente.CDCLIENTE && this.clienteGridCurrentRow.ID_CONTROLA_SLA !== 'N';
	}

	get hasEditedRowsSlaclienteAlertGrid() {
		return !!this.editedRowsSlaclienteAlertGrid.length;
	}

	get hasNoEditedRowsSlaclienteAlertGrid() {
		return !this.editedRowsSlaclienteAlertGrid.length;
	}

	public isEditableColStructureTypeId(paras: any) {
		return paras.data.ID_RESP_PROD_ESTRUT === 'E';
	}

	get isDisableDeleteActionSlaclienteAlertGrid() {
		const instances = Metadata.getInstances('slaclienteAlertGrid');
		if (!instances[0]) {
			return false;
		}
		return this.disableDeleteActionSlaclienteAlertGrid;
	}

	get isDisableAddActionSlaclienteAlertGrid() {
		const currentCliente = Store.getters['Cliente/getCliente'];
		return !!this.editedRowsControlSlaclienteAlertGrid.length || currentCliente.ID_PARAM_SLA === 'P';
	}

	/** Actions  */

	public async reloadSlaclienteAlertGrid() {
		const { datasource } = OMInstance.getInstanceTekAgGrid('slaclienteAlertGrid');
		try {
			this.showLoadingSlaclienteAlertGrid();
			await datasource.get();
			this.disableDeleteActionSlaclienteAlertGrid = true;
		} catch (e) {
			OmDialogService.showErrorBackend(e);
		} finally {
			this.hideLoadingSlaclienteAlertGrid();
		}
	}

	public async saveSlaclienteAlertGrid() {
		try {
			this.showLoadingSlaclienteAlertGrid();
			await ApiSlaclienteAlert.updateSlaclienteAlert(this.editedRowsSlaclienteAlertGrid);
			this.resetEditRowsSlaclienteAlertGrid();
			OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
		} catch (e) {
			OmDialogService.showErrorBackend(e);
		} finally {
			this.hideLoadingSlaclienteAlertGrid();
		}
	}

	public async cancelSlaclienteAlertGrid() {
		await this.reloadSlaclienteAlertGrid();
		this.resetEditRowsSlaclienteAlertGrid();
	}

	public async deleteSlaclienteAlertGrid() {
		const { gridComponent } = OMInstance.getInstanceTekAgGrid('slaclienteAlertGrid');
		const { api: gridApi } = gridComponent.gridOptions;
		const rowsNode = gridApi.getSelectedNodes();
		const rows = rowsNode.map((row: any) => row.data);
		const params = { data: rows };
		console.log(params);
		this.showLoadingSlaclienteGrid();
		try {
			await ApiSlaclienteAlert.deleteSlaclienteAlert(params);
			gridApi.deselectAll();
			OmDialogService.showSuccess(Messages.SUCCESS_DELETE_DEAFULT);
			await this.reloadSlaclienteGrid();
		} catch (e) {
			OmDialogService.showErrorBackend(e);
		} finally {
			this.hideLoadingSlaclienteGrid();
		}
	}

	/** Events  */

	public onRowClickedSlaclienteAlertGrid() {
		console.log();
	}

	public onCellValueChangedSlaclienteAlertGrid(params: any) {
		const id = params.data.ID;
		const rowNode = params.api.getRowNode(params.rowIndex);

		if (params.data.ID_RESP_PROD_ESTRUT === 'P') {
			params.data.STRUCTURE_TYPE_ID = null;
			params.data.STRUCTURE_TYPE_LABEL = '';
			rowNode.setData(params.data);
		}

		if (!this.editedRowsControlSlaclienteAlertGrid.includes(id)) {
			this.editedRowsControlSlaclienteAlertGrid.push(id);
			this.editedRowsSlaclienteAlertGrid.push(params.data);
		} else {
			const index = this.editedRowsControlSlaclienteAlertGrid.indexOf(id);
			this.editedRowsSlaclienteGrid[index] = params.data;
		}
	}

	public onRowSelectedSlaclienteAlertGrid(params: any) {
		this.disableDeleteActionSlaclienteAlertGrid = !params.api.getSelectedNodes().length || this.clienteGridCurrentRow.ID_PARAM_SLA === 'P';
	}

	/** methods */

	public showLoadingSlaclienteAlertGrid() {
		const { gridComponent } = OMInstance.getInstanceTekAgGrid('slaclienteAlertGrid');
		gridComponent.gridOptions.api.showLoadingOverlay();
	}

	public hideLoadingSlaclienteAlertGrid() {
		const { gridComponent } = OMInstance.getInstanceTekAgGrid('slaclienteAlertGrid');
		gridComponent.gridOptions.api.hideOverlay();
	}

	public resetEditRowsSlaclienteAlertGrid() {
		this.editedRowsSlaclienteAlertGrid = [];
		this.editedRowsControlSlaclienteAlertGrid = [];
		this.disableDeleteActionSlaclienteAlertGrid = true;
	}

	public async selectFormatterStructureTypeId(params: any) {
		try {
			if (!this.cachedDataStructureTypeId) {
				const { data: { data } } = await ApiSlaclienteAlert.findStructureType();
				this.cachedDataStructureTypeId = data;
			}

			if (!this.cachedDataStructureTypeId) {
				return params.value;
			}

			const foundRow = this.cachedDataStructureTypeId.find((row: any) => row.ID === params.value);
			if (!foundRow) {
				return params.value;
			}

			return foundRow.LABEL;
		} catch (e) {
			OmDialogService.showErrorBackend(e);
			return params.value;
		}
	}

	/** End Section */

	/** *************************************************************************************
	 ************************ Section Add Modal SLA Cliente Alert **************************
	 ************************************************************************************** */

	/** Modal Methods */

	private addSlaclienteAlertModal!: Modal;

	public async createAddSlaclienteAlertModal() {
		this.addSlaclienteAlertModal = await ModalService.createFromJson('parametrizacaoSlaCliente/modal/addSlaclienteAlert', true);
	}

	public destroyAddSlaclienteAlertModal() {
		this.addSlaclienteAlertModal.destroy();
	}

	public openAddSlaclienteAlertModal() {
		this.addSlaclienteAlertModal.show();
	}

	public closeAddSlaclienteAlertModal() {
		this.addSlaclienteAlertModal.hide();
	}

	/** Form|Inputs Methods */

	public async idRespProdEstrutChangeAddSlaclienteModalForm() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteAlertModalForm');
		const structureTypeId = form.getChildInstance<Select>('STRUCTURE_TYPE_ID');
		if (form.value.ID_RESP_PROD_ESTRUT !== 'E') {
			structureTypeId.removeValidation('required');
			await structureTypeId.setValue('');
			structureTypeId.disabled = true;
		} else {
			structureTypeId.addValidation('required');
			structureTypeId.disabled = false;
		}
	}

	public async saveAddSlaclienteAlertModal() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteAlertModalForm');
		const { CDCLIENTE } = Store.getters['Cliente/getCliente'];
		const { CDGRUPFASE, CDASSUNTO, CDCOMPLEXID } = Store.getters['Cliente/getSlacliente'];
		const row = { ...form.value, CDCLIENTE, CDGRUPFASE, CDASSUNTO, CDCOMPLEXID };
		if (form.validate()) {
			ScreenService.showLoader();
			try {
				await ApiSlaclienteAlert.saveSlaclienteAlert(row);
				this.closeAddSlaclienteAlertModal();
				ScreenService.hideLoader();
				OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
				await this.reloadSlaclienteAlertGrid();
			} catch (e) {
				ScreenService.hideLoader();
				OmDialogService.showErrorBackend(e);
			}
		}
	}

	public keyupSearchFieldSlaclienteAlertGrid({ component }: IEventParam<any>) {
		clearTimeout(this.timeout);
		this.timeout = setTimeout(() => {
			const { datasource } = OMInstance.getInstanceTekAgGrid('slaclienteAlertGrid');
			const { value } = component;
			if (value) {
				datasource.data = datasource.data.filter((row: any) => {
					const idantesdepoisLabel = row.IDANTESDEPOIS_LABEL || '';
					const labelRespProdEstrut = row.LABEL_RESP_PROD_ESTRUT || '';
					const structureTypeLabel = row.STRUCTURE_TYPE_LABEL || '';
					const qthorasalert = row.QTHORASALERT || '';
					return idantesdepoisLabel.toLowerCase().includes(value.toLowerCase())
					|| labelRespProdEstrut.toLowerCase().includes(value.toLowerCase())
					|| structureTypeLabel.toLowerCase().includes(value.toLowerCase())
					|| qthorasalert.toLowerCase().includes(value.toLowerCase());
				});
			} else {
				datasource.data = this.slaclienteAlertGridDataBackup;
			}
		}, 750);
	}

	/** End Section */

	public selectFormatterDatasourceMemory(params: any) {
		const { dataValue, dataText, datasource: { data } } = params.colDef.cellEditorParams.componentProps;
		const foundRow = data.find((row: any) => row[dataValue] === params.value);
		return !foundRow ? params.value : foundRow[dataText[0]];
	}

	private setDatasourceSearchInAllByGrid(value: string, datasource: IDatasource, gridName: string) {
		if (value) {
			datasource.search = value;
			datasource.searchIn = this.getColDefByTeknisaName(gridName);
		} else {
			datasource.search = '';
			datasource.searchIn = [];
		}
	}

	private getColDefByTeknisaName(name: string) {
		const tekGrid = OMInstance.getInstanceTekAgGrid(name);
		return tekGrid.gridComponent.gridOptions.columnDefs.map((row: any) => row.field);
	}

	public async testeChange(params: any) {
		const { datasource } = OMInstance.getInstanceTekAgGrid('clienteGrid');
		datasource.filter.filter = params.api.getFilterModel();
		// let value = filterModel.NMFANTCLIE.filter;
		// value = value ? value.toString() : '';
		// this.setDatasourceSearchInAllByGrid(value, datasource, 'clienteGrid');
		console.log(datasource);
		this.showLoadingClienteGrid();
		try {
			const response = await datasource.get();
			this.hideLoadingClienteGrid();
			return response;
		} catch (error) {
			this.hideLoadingClienteGrid();
			return false;
		}
	}
}
