import Vue from 'vue';
import Router from 'vue-router';
import { Config } from '@zeedhi/core';
import routes from './routes';
import Store from '../store';
import infoModules from '../../../modules.json';

Vue.use(Router);

const router = new Router({
	mode: 'history',
	base: Config.env.baseUrl,
	routes,
});

router.beforeEach((to, from, next) => {
	/** Somente enqunto não logar por conta proria - Início */
	if (window.parent !== window && to.name !== 'token-error' && to.name !== 'microservice') { // if application is called inside a iframe
		if (to.query.token) {
			// if token was passed
			localStorage.setItem('LOGIN_TOKEN', to.query.token.toString());
			localStorage.setItem('LOGIN_USER', to.query.user.toString());
			localStorage.setItem('LOGIN_LANGUAGE', to.query.languange.toString());
			if (to.query.hash !== '0') {
				localStorage.setItem('LOGIN_HASH', to.query.hash.toString());
			}
			Store.commit('setAppLoginToken', to.query.token);
			Store.commit('setAppDisplayOptions', { menu: to.query.menu, header: to.query.header });
		}

		if (!Store.state.appLoginToken) {
			// if there is no token... redirect to another page
			Store.commit('setAppDisplayOptions', { menu: '0', header: '0' });
			next('/token-error');
			return;
		}

		// remove parameters from requested page
		delete to.query.menu;
		delete to.query.header;
		delete to.query.token;

		next();
	}
	/** Somente enqunto não logar por conta proria - Fim */

	if (to.path !== Config.env.loginRoutePath && !localStorage.getItem('LOGIN_TOKEN')) {
		next(Config.env.loginRoutePath);
	} else if (to.path === Config.env.loginRoutePath && localStorage.getItem('LOGIN_TOKEN')) {
		next(Config.env.baseUrl);
	} else {
		next();
	}
});

router.afterEach((to, from) => {
	const htmlElement = document.getElementsByTagName('html')[0];
	if (to.name === 'microservice') {
		htmlElement.setAttribute('microservice', '');
		Store.commit('setRouterParams', to.params);
		const frame = (document.getElementById('microservice-frame') as HTMLIFrameElement);
		if (frame && frame.contentWindow && to.path.split('/')[2] === from.path.split('/')[2]) {
			frame.contentWindow.postMessage({
				type: 'microservice-route-change',
				routerParams: to.params,
			}, `${infoModules[to.path.split('/')[2]].baseUrl}`);
		}
		window.document.title = `${Config.env.product.name} - ${to.params.page}`;
	} else {
		htmlElement.removeAttribute('microservice');
		window.document.title = Config.env.product.name;
	}

	if (to.name === 'login') {
		htmlElement.setAttribute('login', '');
	} else {
		htmlElement.removeAttribute('login');
	}
});

export default router;
